﻿/*
 * Copyright (c) eHealth
 */
using System;
using ikvm.extensions;
using NUnit.Framework;
using java.io;
using java.util;
using org.joda.time;
using org.joda.time.format;
using be.fgov.ehealth.businessconnector.tests.utils;
using be.fgov.ehealth.technicalconnector.tests.utils;
using be.ehealth.technicalconnector.config;
using be.ehealth.technicalconnector.exception;
using be.ehealth.technicalconnector.session;
using be.ehealth.technicalconnector.utils;
using be.fgov.ehealth.standards.kmehr.mycarenet.schema.v1;
using be.fgov.ehealth.technicalconnector.tests.session;
using be.ehealth.business.common.domain;
using be.ehealth.technicalconnector.adapter;
using be.cin.encrypted;
using be.fgov.ehealth.messageservices.mycarenet.core.v1;
using be.ehealth.businessconnector.mycarenet.attestv2.domain;
using be.ehealth.businessconnector.mycarenet.attestv2.session;
using be.fgov.ehealth.mycarenet.attest.protocol.v2;
using be.ehealth.businessconnector.attestv2.helper;

namespace be.ehealth.businessconnector.attestv2.session
{
	/**
	 * eAttest Integration Test. The scenarios numbering matches one specified by mycarenet.
	 *
	 * @author EHP
	 */
	[TestFixture]
	public class CancelAttestDentistIntegrationTest
	{
	
        private AttestIntegrationTestHelper helper = new AttestIntegrationTestHelper();
    
        [SetUp]
        public static  void setupsession() {
            SessionInitializer.init(".\\be.ehealth.businessconnector.attestv2.dentist.test.properties", SupportedLanguages.NET.getAbbreviation());
        }
        
        [TearDown]
        public static void teardownSession()  {
            Session.getInstance().unloadSession();
        }
    
        [Test]
        public void cancelAttestationOA100DENT8a()  {
            InputReference inputReference = new InputReference();
            String scenarioName = "DENT8a";
    
            CancelAttestBuilderRequest attestBuilderRequest = helper.buildRequest(new InputReference(),scenarioName, "73052005540", "DENT","100-1-190226-9851229-88");
            CancelAttestationResponse response = AttestSessionServiceFactory.getAttestService().cancelAttestation(attestBuilderRequest.getCancelAttestationRequest());
    
            assertCancelResult(inputReference, scenarioName, "100", "DENT", response, attestBuilderRequest);
        }
    
       [Test]
        public void cancelAttestationOA500DENT8a()  {
            InputReference inputReference = new InputReference();
            String scenarioName = "DENT8a";
    
            CancelAttestBuilderRequest attestBuilderRequest = helper.buildRequest(new InputReference(),scenarioName, "62110906574", "DENT","500-1-190110-4567890-16");
            CancelAttestationResponse response = AttestSessionServiceFactory.getAttestService().cancelAttestation(attestBuilderRequest.getCancelAttestationRequest());
    
            assertCancelResult(inputReference, scenarioName, "500", "DENT", response, attestBuilderRequest);
        }
    
        [Test]
        public void cancelAttestationOA100DENT8b()  {
            InputReference inputReference = new InputReference();
            String scenarioName = "DENT8b";
    
            CancelAttestBuilderRequest attestBuilderRequest = helper.buildRequest(new InputReference(),scenarioName, "73052005540", "DENT","100-1-190226-9851229-88");
            CancelAttestationResponse response = AttestSessionServiceFactory.getAttestService().cancelAttestation(attestBuilderRequest.getCancelAttestationRequest());
    
            assertCancelResult(inputReference, scenarioName, "100", "DENT", response, attestBuilderRequest);
        }
    
        [Test]
        public void cancelAttestationOA500DENT8b()  {
            InputReference inputReference = new InputReference();
            String scenarioName = "DENT8b";
    
            CancelAttestBuilderRequest attestBuilderRequest = helper.buildRequest(new InputReference(),scenarioName, "62110906574", "DENT","500-1-190110-4567890-16");
            CancelAttestationResponse response = AttestSessionServiceFactory.getAttestService().cancelAttestation(attestBuilderRequest.getCancelAttestationRequest());
    
            assertCancelResult(inputReference, scenarioName, "500", "DENT", response, attestBuilderRequest);
        }
    
        [Test]
        public void cancelAttestationOA100DENT8c()  {
            InputReference inputReference = new InputReference();
            String scenarioName = "DENT8c";
    
            CancelAttestBuilderRequest attestBuilderRequest = helper.buildRequest(new InputReference(),scenarioName, "73052005540", "DENT","100-1-190226-9851229-88");
            CancelAttestationResponse response = AttestSessionServiceFactory.getAttestService().cancelAttestation(attestBuilderRequest.getCancelAttestationRequest());
    
            assertCancelResult(inputReference, scenarioName, "100", "DENT", response, attestBuilderRequest);
        }
    
        [Test]
        public void cancelAttestationOA500DENT8c()  {
            InputReference inputReference = new InputReference();
            String scenarioName = "DENT8c";
    
            CancelAttestBuilderRequest attestBuilderRequest = helper.buildRequest(new InputReference(),scenarioName, "62110906574", "DENT","500-1-190110-4567890-16");
            CancelAttestationResponse response = AttestSessionServiceFactory.getAttestService().cancelAttestation(attestBuilderRequest.getCancelAttestationRequest());
    
            assertCancelResult(inputReference, scenarioName, "500", "DENT", response, attestBuilderRequest);
        }
    
        [Test]
        public void cancelAttestationOA100DENT9()  {
          InputReference inputReference = new InputReference();
          String ssin = "73052005540";
          SendAttestBuilderRequest sendBuilderRequest = helper.buildRequest(inputReference,"DENT4", ssin, "DENT");
          SendAttestationResponse sendResponse = AttestSessionServiceFactory.getAttestService()
                    .sendAttestation(sendBuilderRequest.getSendAttestationRequest());
          String invoiceNumber = helper.getInvoiceNumber(sendResponse, sendBuilderRequest);      
          String scenarioName = "DENT9";
          CancelAttestBuilderRequest cancelBuilderRequest = helper.buildRequest(new InputReference(),scenarioName, "73052005540", "DENT",invoiceNumber);
          CancelAttestationResponse response = AttestSessionServiceFactory.getAttestService().cancelAttestation(cancelBuilderRequest.getCancelAttestationRequest());
          
          assertCancelResult(inputReference, scenarioName, "100", "DENT", response, cancelBuilderRequest);
        }
    
        [Test]
        public void cancelAttestationOA500DENT9()  {
          InputReference inputReference = new InputReference();
          String ssin = "62110906574";
          SendAttestBuilderRequest sendBuilderRequest = helper.buildRequest(inputReference,"DENT9Send", ssin, "DENT");
          SendAttestationResponse sendResponse = AttestSessionServiceFactory.getAttestService()
                    .sendAttestation(sendBuilderRequest.getSendAttestationRequest());
          String invoiceNumber = helper.getInvoiceNumber(sendResponse, sendBuilderRequest);
          String scenarioName = "DENT9";
          CancelAttestBuilderRequest cancelBuilderRequest = helper.buildRequest(new InputReference(),scenarioName, ssin, "DENT",invoiceNumber);
          CancelAttestationResponse response = AttestSessionServiceFactory.getAttestService().cancelAttestation(cancelBuilderRequest.getCancelAttestationRequest());

          assertCancelResult(inputReference, scenarioName, "500", "DENT", response, cancelBuilderRequest);
        }
    
        private void assertCancelResult(InputReference inputReference, String scenarioName, String oa, String profession, CancelAttestationResponse response, CancelAttestBuilderRequest cancelAttestBuilderRequest) {
        	SignedBuilderResponse builderResponse = helper.getBuildCancelAttestResponse(response, cancelAttestBuilderRequest.getCancelAttestationRequest());
	        Assert.AreEqual(0, builderResponse.getSignatureVerificationResult().getErrors().size(), "SignatureVerificationResult should contain no error");
	        // check signedData is same as content of the business blob in the WS reponse
	        Assert.AreEqual(response.getReturn().getDetail().getValue(), builderResponse.getSignedData());
	        // check xadesT is same as the one from the WS reponse
	        Assert.AreEqual(response.getReturn().getXadesT().getValue(), builderResponse.getXadesT());      
	        String expectedResponse = helper.buildExpectedCancelAttestResponse(builderResponse, inputReference.getInputReference(),scenarioName, oa, profession);
	        be.fgov.ehealth.businessconnector.tests.utils.XmlAsserter.assertSimilar(expectedResponse, ConnectorXmlUtils.toString(builderResponse.getSendTransactionResponse()));
    }
    
	}
}
